@@ -11,20 +11,13 @@ package NCGI::Query;
 # ----------------------------------------------------------------------
 use strict;
 use warnings;
-use base 'Class::Singleton';
+use base 'NCGI::Singleton';
 use NCGI::Cookie;
+use CGI::Util qw(unescape);
+use debug;
 
 our $VERSION = '0.01';
 
-sub new {
-    my $proto = shift;
-    my $class = ref($proto) || $proto;
-    no strict 'refs';
-    my $instance = \${ "$class\::_instance" };
-    undef $$instance;
-    return $class->instance(@_);
-}
-
 #
 # This only gets called from the first Class::Singleton::instance() call
 #
@@ -32,17 +25,18 @@ sub _new_instance {
     my $proto = shift;
     my $class = ref($proto) || $proto;
     my $self  = {
-        debug => 0,
         @_,
     };
 
-    $self->{q_query} = {};
-    $self->{q_full}  = '';
-    $self->{q_type}  = 'NOQUERY';
+    $self->{q_params} = {};
+    $self->{q_full}   = '';
+    $self->{q_type}   = 'NOQUERY';
+
 
     if ($ENV{CONTENT_LENGTH}) {
         $self->{q_type}  = 'POST';
-        if (!defined(read(STDIN, $self->{q_full}, $ENV{CONTENT_LENGTH}))) {
+        my $length = read(STDIN, $self->{q_full}, $ENV{CONTENT_LENGTH});
+        if (!defined($length) or $length != $ENV{CONTENT_LENGTH}) {
             warn "Could not read from STDIN: $!";
             return undef;
         }
@@ -53,19 +47,20 @@ sub _new_instance {
     }
     chomp($self->{q_full});
 
-    $self->{q_query} = {};
+    $self->{q_params} = {};
     foreach (split(/[&;]/, $self->{q_full})) {
         my ($key, $val) = split('=', $_, 2);
-        $self->{q_query}->{unescape($key)} = unescape($val);
+        $self->{q_params}->{unescape($key)} = unescape($val);
     }
 
     $self->{q_cookies} = NCGI::Cookie::fetch();
 
+    debug::log('NCGI::Query Initialised with '. $self->{q_type}) if(DEBUG);
+
     bless ($self, $class);
     return $self;
 }
 
-
 sub isquery {
     my $self = shift;
     return ($self->{q_type} ne 'NOQUERY');
@@ -81,9 +76,16 @@ sub ispost {
     return ($self->{q_type} eq 'POST');
 }
 
-sub query {
+sub param {
     my $self = shift;
-    return $self->{q_query};
+    my $param = shift;
+    exists($self->{q_params}->{$param}) && return $self->{q_params}->{$param};
+    return undef;
+}
+
+sub params {
+    my $self = shift;
+    return $self->{q_params};
 }
 
 sub full_query {
@@ -93,9 +95,34 @@ sub full_query {
 
 sub cookie {
     my $self = shift;
-    return $self->{q_cookie};
+    my $param = shift;
+    exists($self->{q_cookies}->{$param}) && return $self->{q_cookies}->{$param};
+    return undef;
 }
 
+sub cookies {
+    my $self = shift;
+    return $self->{q_cookies};
+}
+
+sub dump_params {
+    my $self = shift;
+    my $str = '';
+    while (my ($key, $val) = each %{$self->{q_params}}) {
+        $val = '******' if ($key =~ /pass/);
+        $str .= "$key = $val\n";
+    }
+    return $str;
+}
+
+sub dump_cookies {
+    my $self = shift;
+    my $str;
+    while (my ($key, $val) = each %{$self->{q_cookies}}) {
+        $str .= "$key = $val\n";
+    }
+    return $str;
+}
 
 1;
 __END__
